@@ -43,6 +43,12 @@ RcppExport SEXP loglik_AtEt_epsp_c(SEXP v_b_a, SEXP v_b_e, SEXP pheno_m, SEXP ph
 	k_1d << 0.5 << 0 << arma::endr << 0 << 0.5 << arma::endr;
 	arma::vec  v = arma::ones<arma::vec>(2);
 	arma::mat diag = arma::diagmat(v);
+	int penal_a = 2;
+	if(d_a(0,1)==-1)
+	{penal_a = 1;}
+	int penal_e = 2;
+	if(d_e(0,1)==-1)
+	{penal_e = 1;}
 	
 	k_m.fill(1);
 	k_2d.fill(0.5);
@@ -180,7 +186,7 @@ RcppExport SEXP loglik_AtEt_epsp_c(SEXP v_b_a, SEXP v_b_e, SEXP pheno_m, SEXP ph
 
 	double gsd = log(fabs(det(0.5*gsd_max)));
 
-	double res = D_m + YSY_m + D_d + YSY_d + (num_a-2)*log(va) + (num_e-2)*log(ve) + as_scalar(trans(ba)*d_a*ba)/va + as_scalar(trans(be)*d_e*be)/ve + gsd;
+	double res = D_m + YSY_m + D_d + YSY_d + (num_a-penal_a)*log(va) + (num_e-penal_e)*log(ve) + as_scalar(trans(ba)*d_a*ba)/va + as_scalar(trans(be)*d_e*be)/ve + gsd;
 
 	return(Rcpp::wrap(res));
 }
@@ -227,7 +233,8 @@ RcppExport SEXP loglik_AtEt_epsp_g_c(SEXP b_a, SEXP b_e, SEXP pheno_m, SEXP phen
 	{
 		int start = 2*i;
 		int end = start + 1;
-		arma::mat V_m = exp(as_scalar(b_e_m.row(start)*be))*diag+exp(as_scalar(b_a_m.row(start)*ba))*k_m;
+		double be_m = exp(as_scalar(b_e_m.row(start)*be));
+		arma::mat V_m = be_m*diag+exp(as_scalar(b_a_m.row(start)*ba))*k_m;
 		double det_m = V_m(0,0)*V_m(0,0)-V_m(0,1)*V_m(0,1);
 		//inv_V_m = inv(V_m);
 		arma::mat inv_V_m = V_m/det_m;
@@ -246,7 +253,8 @@ RcppExport SEXP loglik_AtEt_epsp_g_c(SEXP b_a, SEXP b_e, SEXP pheno_m, SEXP phen
 	{
 		int start = 2*i;
 		int end = start + 1;
-		arma::mat V_d = exp(as_scalar(b_e_d.row(start)*be))*diag+exp(as_scalar(b_a_d.row(start)*ba))*(k_1d+k_2d);
+		double be_d = exp(as_scalar(b_e_d.row(start)*be));
+		arma::mat V_d = be_d*diag+exp(as_scalar(b_a_d.row(start)*ba))*(k_1d+k_2d);
 		//arma::mat inv_V_d = inv(V_d);
 		arma::mat inv_V_d(2,2);
 		double det_d = V_d(0,0)*V_d(0,0)-V_d(0,1)*V_d(0,1);
@@ -262,7 +270,11 @@ RcppExport SEXP loglik_AtEt_epsp_g_c(SEXP b_a, SEXP b_e, SEXP pheno_m, SEXP phen
 		D_d = D_d + log(det_d);
 	}
 
-	double res = D_m + YSY_m + D_d + YSY_d + as_scalar(trans(ba)*d_a*ba)/va + as_scalar(trans(be)*d_e*be)/ve;
+	double res = D_m + YSY_m + D_d + YSY_d;
+	if(va>0)
+		res += as_scalar(trans(ba)*d_a*ba)/va;
+	if(ve>0)
+		res += as_scalar(trans(be)*d_e*be)/ve;
 
 	return(Rcpp::wrap(res));
 }
@@ -361,8 +373,12 @@ RcppExport SEXP gr_AtEt_epsp_g_c(SEXP b_a, SEXP b_e, SEXP pheno_m, SEXP pheno_d,
 	}
 
 	arma::vec res(num_a+num_e);
-	arma::vec res_a = v_b_a_m + v_b_a_d + 2*d_a*ba/va;
-	arma::vec res_e = v_b_e_m + v_b_e_d + 2*d_e*be/ve;
+	arma::vec res_a = v_b_a_m + v_b_a_d;
+	if(va>0)
+		res_a += 2*d_a*ba/va;
+	arma::vec res_e = v_b_e_m + v_b_e_d;
+	if(ve>0)
+		res_e += 2*d_e*be/ve;
 	for(int i = 0; i < num_a; i++)
 	{
 		res(i) = res_a(i);
@@ -401,6 +417,12 @@ RcppExport SEXP gr_AtEt_epsp_c(SEXP v_b_a, SEXP v_b_e, SEXP pheno_m, SEXP pheno_
 	k_1d << 0.5 << 0 << arma::endr << 0 << 0.5 << arma::endr;
 	arma::vec  v = arma::ones<arma::vec>(2);
 	arma::mat diag = arma::diagmat(v);
+	int penal_a = 2;
+	if(d_a(0,1)==-1)
+	{penal_a = 1;}
+	int penal_e = 2;
+	if(d_e(0,1)==-1)
+	{penal_e = 1;}
 	
 	k_m.fill(1);
 	k_2d.fill(0.5);
@@ -547,9 +569,9 @@ RcppExport SEXP gr_AtEt_epsp_c(SEXP v_b_a, SEXP v_b_e, SEXP pheno_m, SEXP pheno_
 	gsd_b_e.submat(num_a,num_a,num_a+num_e-1,num_a+num_e-1) = arma::diagmat(2*d_e.diag());
 
 	arma::mat temp_m = inv_gsd_max*gsd_b_a;
-	double d_v_b_a = ((-1)*as_scalar(trans(ba)*d_a*ba)-0.5*arma::sum(temp_m.diag()))/(va*va) + (num_a-2)/va;
+	double d_v_b_a = ((-1)*as_scalar(trans(ba)*d_a*ba)-0.5*arma::sum(temp_m.diag()))/(va*va) + (num_a-penal_a)/va;
 	temp_m = inv_gsd_max*gsd_b_e;
-	double d_v_b_e = ((-1)*as_scalar(trans(be)*d_e*be)-0.5*arma::sum(temp_m.diag()))/(ve*ve) + (num_e-2)/ve;
+	double d_v_b_e = ((-1)*as_scalar(trans(be)*d_e*be)-0.5*arma::sum(temp_m.diag()))/(ve*ve) + (num_e-penal_e)/ve;
 
 	arma::vec res(2);
 	res(0) = d_v_b_a;
